#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"

#include <nanova.h>

#include "const-c.inc"

#define HTXT "NAnova::UnbalancedNestedANOVA: "

MODULE = NAnova		PACKAGE = NAnova		

INCLUDE: const-xs.inc

AV *
UnbalancedNestedANOVA(tree, groups, values)
	AV * tree
	AV * groups
	AV * values
    PREINIT:
	unsigned int nb_levels, nb_groups, nb_values;
	unsigned int *t=NULL;
	unsigned int *g=NULL;
    	double *v=NULL;
	int i, j;
	SV **sv;
    CODE:
	nb_groups=av_len(groups)+1;
	nb_values=av_len(values)+1;
	if (nb_groups <= 0) {
		fprintf(stderr, HTXT "no group to ananyze\n");
		XSRETURN_UNDEF;
	}
	if (av_len(tree)+1 != nb_groups) {
		fprintf(stderr, 
			HTXT "tree and groups are not the same size!\n");
		XSRETURN_UNDEF;
	}
	sv=av_fetch(tree, 0, 0);
	if (!SvROK(*sv) || SVt_PVAV!=SvTYPE(SvRV(*sv))) {
		fprintf(stderr, HTXT 
			"tree[%i] is not an array reference\n", i);
		goto error;
	}
	nb_levels=av_len((AV*)SvRV(*sv))+1;
	
	g=malloc(sizeof(int)*nb_groups);
	t=malloc(sizeof(int)*nb_groups*nb_levels);
	v=malloc(sizeof(double)*nb_values);
	if (g==NULL || t==NULL || v==NULL) {
		fprintf(stderr, HTXT "cannot allocate memory\n");
		goto error;
	}
	for (i=0; i<nb_groups; i++) {
		AV * levels;
		sv=av_fetch(tree, i, 0);
		if (!SvROK(*sv) || SVt_PVAV!=SvTYPE(SvRV(*sv))) {
			fprintf(stderr, HTXT 
				"tree[%i] is not an array reference\n", i);
			goto error;
		}
		levels=(AV *)SvRV(*sv);
		if (av_len(levels)+1 != nb_levels) {
			fprintf(stderr, HTXT 
				"@{tree[%i]} does not have %i elements\n",
				i, nb_levels);
			goto error;
		}
		for (j=0; j<nb_levels; j++) {
			sv=av_fetch(levels, j, 0);
			if (!SvIOK(*sv)) {
				fprintf(stderr, HTXT 
				    "tree[%i]->[%i] is not an unsigned int\n",
				    i, j);
				goto error;
			}
			t[i*nb_levels+j]=SvIV(*sv);
		}
	}
	for (i=0; i<nb_groups; i++) {
		sv=av_fetch(groups, i, 0);
		if (!SvIOK(*sv)) {
			fprintf(stderr, HTXT
				"groups[%i] is not an unsigned int\n", i);
			goto error;
		}
		g[i]=SvIV(*sv);
	}
	for (i=0; i<nb_values; i++) {
		sv=av_fetch(values, i, 0);
		if (!SvNOK(*sv) && !SvIOK(*sv)) {
			fprintf(stderr, HTXT "values[%i] is not an double\n",
				i);
			goto error;
		}
		v[i]=SvNV(*sv);
	}
	{
		struct una_results results[nb_levels];
		int ret;
		ret=unbalanced_nested_anova(nb_levels, nb_groups, nb_values,
			t, g, v, (struct una_results*)&results);

		free(v);
		free(t);
		free(g);
		if (ret) {
			fprintf(stderr, HTXT "error: return value %i\n", ret);
			XSRETURN_UNDEF;
		}

		RETVAL = newAV();
		sv_2mortal((SV *)RETVAL);
		for(i=0; i<nb_levels; i++) {
			HV* hv=(HV *)sv_2mortal((SV *)newHV());
			hv_store(hv, "p_value", 7, 
					newSVnv(results[i].p_value), 0);
			hv_store(hv, "F", 1, 
					newSVnv(results[i].F), 0);
			av_push(RETVAL, newRV((SV *)hv));
		}
		
	}
	goto end;
	error:
		free(v);
		free(t);
		free(g);
		XSRETURN_UNDEF;
	end:
    OUTPUT:
	RETVAL
		
	
